@@ -1,4 +1,3 @@
-use crate::alloc::borrow::ToOwned;
 use crate::traits::FromPoints;
 use alloc::vec;
 use alloc::vec::Vec;
@@ -79,7 +78,7 @@ impl Stream {
     /// use screech::traits::FromPoints;
     /// use screech::stream::Stream;
     ///
-    /// let stream = Stream::from_points(&[0.0, 0.1, 0.2]);
+    /// let stream = Stream::from_points(vec![0.0, 0.1, 0.2]);
     ///
     /// assert_eq!(stream.get_point(1), Some(&0.1));
     /// assert_eq!(stream.get_point(10), None);
@@ -97,14 +96,14 @@ impl Stream {
     /// use screech::traits::FromPoints;
     /// use screech::stream::Stream;
     ///
-    /// let stream = Stream::from_points(&[0.0, 0.1, 0.2]);
+    /// let stream = Stream::from_points(vec![0.0, 0.1, 0.2]);
     ///
-    /// assert_eq!(stream.get_points(), vec![0.0, 0.1, 0.2]);
+    /// assert_eq!(stream.get_points().unwrap(), &[0.0, 0.1, 0.2]);
     /// ```
-    pub fn get_points(&self) -> Vec<Point> {
+    pub fn get_points(&self) -> Option<&Vec<Point>> {
         match self {
-            Stream::Points(points) => points.clone(),
-            Stream::Fixed(point) => vec![point.clone()],
+            Stream::Points(points) => Some(points),
+            Stream::Fixed(_) => None,
         }
     }
 
@@ -118,14 +117,14 @@ impl Stream {
     /// use screech::stream::Stream;
     ///
     /// let streams = [
-    ///     &Stream::from_points(&[0.1, 0.0, -0.1, -0.2, -0.3]),
-    ///     &Stream::from_points(&[0.2, 0.1, 0.0]),
-    ///     &Stream::from_points(&[0.3]),
+    ///     &Stream::from_points(vec![0.1, 0.0, -0.1, -0.2, -0.3]),
+    ///     &Stream::from_points(vec![0.2, 0.1, 0.0]),
+    ///     &Stream::from_points(vec![0.3]),
     /// ];
     ///
     /// let result = Stream::mix(&streams);
     ///
-    /// assert_eq!(result.get_points(), vec![0.6, 0.1, -0.1, -0.2, -0.3]);
+    /// assert_eq!(result.get_points().unwrap(), &[0.6, 0.1, -0.1, -0.2, -0.3]);
     /// ```
     pub fn mix(streams: &[&Stream]) -> Self {
         let length = streams.iter().fold(0, |a, b| cmp::max(a, b.len()));
@@ -139,7 +138,42 @@ impl Stream {
             points.push(point);
         }
 
-        Stream::from_points(&points)
+        Stream::from_points(points)
+    }
+
+    /// Same as mix, but mixes sources into the current stream
+    ///
+    /// **note** the size of the stream will be unchanged
+    ///
+    /// ```
+    /// use screech::traits::FromPoints;
+    /// use screech::stream::Stream;
+    ///
+    /// let streams = [
+    ///     &Stream::from_points(vec![0.2, 0.1, 0.0]),
+    ///     &Stream::from_points(vec![0.3]),
+    /// ];
+    ///
+    /// let mut stream = Stream::from_points(vec![0.1, 0.0, -0.1, -0.2, -0.3])
+    ///     .mix_into(&streams);
+    ///
+    /// assert_eq!(stream.get_points().unwrap(), &[0.6, 0.1, -0.1, -0.2, -0.3]);
+    /// ```
+    pub fn mix_into(self, streams: &[&Stream]) -> Self {
+        match self {
+            Stream::Fixed(point) => Stream::Fixed(point),
+            Stream::Points(mut points) => {
+                for (pos, point) in points.iter_mut().enumerate() {
+                    let sum = streams
+                        .iter()
+                        .fold(0.0, |xs, x| xs + x.get_point(pos).unwrap_or(&0.0));
+
+                    *point += sum;
+                }
+
+                Stream::Points(points)
+            }
+        }
     }
 
     /// Map values inside stream
@@ -148,10 +182,10 @@ impl Stream {
     /// use screech::traits::FromPoints;
     /// use screech::stream::Stream;
     ///
-    /// let stream = Stream::from_points(&[0.1, 0.2, 0.3])
+    /// let stream = Stream::from_points(vec![0.1, 0.2, 0.3])
     ///     .map(|point| point * 2.0);
     ///
-    /// assert_eq!(stream.get_points(), &[0.2, 0.4, 0.6]);
+    /// assert_eq!(stream.get_points().unwrap(), &[0.2, 0.4, 0.6]);
     /// ```
     pub fn map<F>(self, f: F) -> Self
     where
@@ -159,12 +193,11 @@ impl Stream {
     {
         match self {
             Stream::Fixed(point) => Stream::Fixed(f(point)),
-            Stream::Points(points) => {
-                let mut mapped_points = vec![];
-                for p in points {
-                    mapped_points.push(f(p));
+            Stream::Points(mut points) => {
+                for point in points.iter_mut() {
+                    *point = f(*point);
                 }
-                Stream::Points(mapped_points)
+                Stream::Points(points)
             }
         }
     }
@@ -175,29 +208,37 @@ impl Stream {
     /// use screech::traits::FromPoints;
     /// use screech::stream::Stream;
     ///
-    /// let stream = Stream::from_points(&[0.1, 0.2, 0.3, 0.4]);
-    /// let gate_cv = Stream::from_points(&[1.0, 0.0, 1.0, 0.0]);
+    /// let stream = Stream::from_points(vec![0.1, 0.2, 0.3, 0.4]);
+    /// let gate_cv = Stream::from_points(vec![1.0, 0.0, 1.0, 0.0]);
     ///
     /// let result = stream.apply(&gate_cv, |signal_point, cv_point| {
     ///     if cv_point == 1.0 { signal_point } else { 0.0 }
     /// });
-    /// assert_eq!(result.get_points(), vec![0.1, 0.0, 0.3, 0.0]);
+    /// assert_eq!(result.get_points().unwrap(), &[0.1, 0.0, 0.3, 0.0]);
     /// ```
     pub fn apply<F>(self, stream: &Self, f: F) -> Self
     where
         F: Fn(Point, Point) -> Point,
     {
-        let mut points = vec![];
+        match (&self, stream) {
+            (Stream::Fixed(a), Stream::Fixed(b)) => Stream::fixed(f(*a, *b)),
+            _ => {
+                let mut points = match self {
+                    Stream::Fixed(point) => vec![point],
+                    Stream::Points(points) => points,
+                };
 
-        let length = cmp::max(self.len(), stream.len());
+                let length = cmp::max(points.len(), stream.len());
+                points.resize(length, 0.0);
 
-        for pos in 0..length {
-            let a = self.get_point(pos).unwrap_or(&0.0);
-            let b = stream.get_point(pos).unwrap_or(&0.0);
-            points.push(f(*a, *b));
+                for (pos, point) in points.iter_mut().enumerate() {
+                    let b = stream.get_point(pos).unwrap_or(&0.0);
+                    *point = f(*point, *b);
+                }
+
+                Stream::Points(points)
+            }
         }
-
-        Stream::from_points(&points)
     }
 
     /// Amplify a stream by decibels
@@ -209,9 +250,9 @@ impl Stream {
     /// use screech::stream::Stream;
     ///
     /// // 6 dBs should roughly double / half
-    /// let stream = Stream::from_points(&[0.1, 0.3, 0.6, -0.1, -0.4, -0.8]).amplify(6.0);
+    /// let stream = Stream::from_points(vec![0.1, 0.3, 0.6, -0.1, -0.4, -0.8]).amplify(6.0);
     ///
-    /// assert_eq!(stream.get_points(), vec![0.19952624, 0.59857875, 1.0, -0.19952624, -0.79810494, -1.0]);
+    /// assert_eq!(stream.get_points().unwrap(), &[0.19952624, 0.59857875, 1.0, -0.19952624, -0.79810494, -1.0]);
     /// ```
     pub fn amplify(self, db: f32) -> Self {
         let ratio = powf(10.0, db / 20.0);
@@ -226,11 +267,11 @@ impl Stream {
     /// use screech::traits::FromPoints;
     /// use screech::stream::Stream;
     ///
-    /// let cv = Stream::from_points(&[1.0, -1.0, 1.0, -1.0]);
-    /// let stream = Stream::from_points(&[0.1, 0.1, 0.1, 0.1, 0.1, 0.1])
+    /// let cv = Stream::from_points(vec![1.0, -1.0, 1.0, -1.0]);
+    /// let stream = Stream::from_points(vec![0.1, 0.1, 0.1, 0.1, 0.1, 0.1])
     ///     .amplify_with_cv(&cv, |p| p * 6.0); // amplify between -6dB and +6dB
     ///
-    /// assert_eq!(stream.get_points(), vec![0.19952624, 0.050118722, 0.19952624, 0.050118722, 0.1, 0.1]);
+    /// assert_eq!(stream.get_points().unwrap(), &[0.19952624, 0.050118722, 0.19952624, 0.050118722, 0.1, 0.1]);
     /// ```
     pub fn amplify_with_cv<F>(self, cv: &Self, point_to_db: F) -> Self
     where
@@ -249,9 +290,9 @@ impl Stream {
     /// use screech::traits::FromPoints;
     /// use screech::stream::Stream;
     ///
-    /// let stream = Stream::from_points(&[0.3, 0.2, 0.1, 0.0, -0.1, -0.2]).invert();
+    /// let stream = Stream::from_points(vec![0.3, 0.2, 0.1, 0.0, -0.1, -0.2]).invert();
     ///
-    /// assert_eq!(stream.get_points(), vec![-0.3, -0.2, -0.1, 0.0, 0.1, 0.2]);
+    /// assert_eq!(stream.get_points().unwrap(), &[-0.3, -0.2, -0.1, 0.0, 0.1, 0.2]);
     /// ```
     pub fn invert(self) -> Self {
         self.map(|point| -point)
@@ -263,11 +304,11 @@ impl Stream {
     /// use screech::traits::FromPoints;
     /// use screech::stream::Stream;
     ///
-    /// let stream = Stream::from_points(&[0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8])
+    /// let stream = Stream::from_points(vec![0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8])
     ///     .slice(2, 3)
     ///     .unwrap();
     ///
-    /// assert_eq!(stream.get_points(), vec![0.3, 0.4, 0.5]);
+    /// assert_eq!(stream.get_points().unwrap(), &[0.3, 0.4, 0.5]);
     /// ```
     pub fn slice(self, from: usize, length: usize) -> Result<Self, StreamErr> {
         let to = from + length;
@@ -279,7 +320,7 @@ impl Stream {
                 if from > length || to > length || from > to {
                     Err(StreamErr::SliceOutOfBounds)
                 } else {
-                    Ok(Stream::from_points(&points[from..to].to_vec()))
+                    Ok(Stream::from_points(points[from..to].to_vec()))
                 }
             }
         }
@@ -291,10 +332,10 @@ impl Stream {
     /// use screech::traits::FromPoints;
     /// use screech::stream::Stream;
     ///
-    /// let stream = Stream::from_points(&[0.1, 0.2, 0.3])
+    /// let stream = Stream::from_points(vec![0.1, 0.2, 0.3])
     ///     .looped_slice(0, 8);
     ///
-    /// assert_eq!(stream.get_points(), vec![0.1, 0.2, 0.3, 0.1, 0.2, 0.3, 0.1, 0.2]);
+    /// assert_eq!(stream.get_points().unwrap(), &[0.1, 0.2, 0.3, 0.1, 0.2, 0.3, 0.1, 0.2]);
     /// ```
     pub fn looped_slice(self, from: usize, length: usize) -> Self {
         let stream_length = self.len();
@@ -302,17 +343,17 @@ impl Stream {
 
         for index in 0..length {
             let pos = (index + from) % stream_length;
-            points.push(self.get_point(pos).unwrap().clone());
+            points.push(*self.get_point(pos).unwrap());
         }
 
-        Stream::from_points(&points)
+        Stream::from_points(points)
     }
 }
 
 impl FromPoints<u8, Stream> for Stream {
     /// Create new stream based on u8 points,
     /// converts u8 to point value (f32 between -1.0 and 1.0)
-    fn from_points(points: &[u8]) -> Stream {
+    fn from_points(points: Vec<u8>) -> Stream {
         Stream::Points(points.iter().copied().map(u8_to_point).collect())
     }
 }
@@ -320,7 +361,7 @@ impl FromPoints<u8, Stream> for Stream {
 impl FromPoints<i16, Stream> for Stream {
     /// Create new stream based on i16 points,
     /// converts i16 to point value (f32 between -1.0 and 1.0)
-    fn from_points(points: &[i16]) -> Stream {
+    fn from_points(points: Vec<i16>) -> Stream {
         Stream::Points(points.iter().copied().map(i16_to_point).collect())
     }
 }
@@ -328,16 +369,16 @@ impl FromPoints<i16, Stream> for Stream {
 impl FromPoints<i32, Stream> for Stream {
     /// Create new stream based on i32 points,
     /// converts i32 to point value (f32 between -1.0 and 1.0)
-    fn from_points(points: &[i32]) -> Stream {
+    fn from_points(points: Vec<i32>) -> Stream {
         Stream::Points(points.iter().copied().map(i32_to_point).collect())
     }
 }
 
 impl FromPoints<f32, Stream> for Stream {
     /// Create new stream based on f32 points
-    fn from_points(points: &[f32]) -> Stream {
+    fn from_points(points: Vec<f32>) -> Stream {
         // @TODO: clamp values?
-        Stream::Points(points.to_owned())
+        Stream::Points(points)
     }
 }
 
@@ -364,59 +405,62 @@ mod tests {
     #[test]
     fn test_mix() {
         let stream = Stream::mix(&[
-            &Stream::from_points(&[1.0, 0.2, 1.0, 1.0, 0.2]),
-            &Stream::from_points(&[0.0, 0.0, 0.0, 0.0, 0.0]),
+            &Stream::from_points(vec![1.0, 0.2, 1.0, 1.0, 0.2]),
+            &Stream::from_points(vec![0.0, 0.0, 0.0, 0.0, 0.0]),
         ]);
 
-        assert_eq!(stream.get_points(), vec![1.0, 0.2, 1.0, 1.0, 0.2]);
+        assert_eq!(stream.get_points().unwrap(), &[1.0, 0.2, 1.0, 1.0, 0.2]);
 
         let stream = Stream::mix(&[
-            &Stream::from_points(&[0.1, 0.0, -0.1, -0.2, -0.3]),
-            &Stream::from_points(&[0.2, 0.1, 0.0, -0.1, -0.2]),
-            &Stream::from_points(&[0.3, 0.2, 0.1, 0.0, -0.1]),
+            &Stream::from_points(vec![0.1, 0.0, -0.1, -0.2, -0.3]),
+            &Stream::from_points(vec![0.2, 0.1, 0.0, -0.1, -0.2]),
+            &Stream::from_points(vec![0.3, 0.2, 0.1, 0.0, -0.1]),
         ]);
 
-        assert_eq!(stream.get_points(), vec![0.6, 0.3, 0.0, -0.3, -0.6]);
+        assert_eq!(stream.get_points().unwrap(), &[0.6, 0.3, 0.0, -0.3, -0.6]);
 
         let stream = Stream::mix(&[
-            &Stream::from_points(&[0.1, 0.0, -0.1, -0.2, -0.3]),
-            &Stream::from_points(&[0.2, 0.1, 0.0]),
-            &Stream::from_points(&[0.3]),
+            &Stream::from_points(vec![0.1, 0.0, -0.1, -0.2, -0.3]),
+            &Stream::from_points(vec![0.2, 0.1, 0.0]),
+            &Stream::from_points(vec![0.3]),
         ]);
 
-        assert_eq!(stream.get_points(), vec![0.6, 0.1, -0.1, -0.2, -0.3]);
+        assert_eq!(stream.get_points().unwrap(), &[0.6, 0.1, -0.1, -0.2, -0.3]);
     }
 
     #[test]
     fn test_amplify() {
         let stream = Stream::empty(1).amplify(6.0);
-        assert_eq!(stream.get_points(), vec![0.0]);
+        assert_eq!(stream.get_points().unwrap(), &[0.0]);
 
         // 6 dBs should roughly double / half
-        let stream = Stream::from_points(&[0.1, 0.25, 0.3, -0.1, -0.4]).amplify(6.0);
+        let stream = Stream::from_points(vec![0.1, 0.25, 0.3, -0.1, -0.4]).amplify(6.0);
 
         let rounded_points: Vec<Point> = stream
             .get_points()
+            .unwrap()
             .iter()
             .map(|x| (x * 10.0).round() / 10.0)
             .collect::<Vec<Point>>();
 
         assert_eq!(rounded_points, vec![0.2, 0.5, 0.6, -0.2, -0.8]);
 
-        let stream = Stream::from_points(&[0.4, 0.5, 0.8, -0.3, -0.6]).amplify(-6.0);
+        let stream = Stream::from_points(vec![0.4, 0.5, 0.8, -0.3, -0.6]).amplify(-6.0);
 
         let rounded_points: Vec<Point> = stream
             .get_points()
+            .unwrap()
             .iter()
             .map(|x| (x * 100.0).round() / 100.0)
             .collect::<Vec<Point>>();
         assert_eq!(rounded_points, vec![0.2, 0.25, 0.4, -0.15, -0.3]);
 
         // clamp the value
-        let stream = Stream::from_points(&[0.1, 0.4, 0.6, -0.2, -0.3, -0.5]).amplify(12.0);
+        let stream = Stream::from_points(vec![0.1, 0.4, 0.6, -0.2, -0.3, -0.5]).amplify(12.0);
 
         let rounded_points: Vec<Point> = stream
             .get_points()
+            .unwrap()
             .iter()
             .map(|x| (x * 100.0).round() / 100.0)
             .collect::<Vec<Point>>();
@@ -425,39 +469,39 @@ mod tests {
 
     #[test]
     fn test_slice() {
-        let stream = Stream::from_points(&[0.0, 0.1, 0.2, 0.3, 0.4, 0.5]);
+        let stream = Stream::from_points(vec![0.0, 0.1, 0.2, 0.3, 0.4, 0.5]);
         assert_eq!(stream.slice(0, 2).unwrap(), Stream::Points(vec![0.0, 0.1]));
 
-        let stream = Stream::from_points(&[0.0, 0.1, 0.2, 0.3, 0.4, 0.5]);
+        let stream = Stream::from_points(vec![0.0, 0.1, 0.2, 0.3, 0.4, 0.5]);
         assert_eq!(
             stream.slice(2, 3).unwrap(),
             Stream::Points(vec![0.2, 0.3, 0.4])
         );
 
-        let stream = Stream::from_points(&[0.0, 0.1, 0.2, 0.3, 0.4, 0.5]);
+        let stream = Stream::from_points(vec![0.0, 0.1, 0.2, 0.3, 0.4, 0.5]);
         let error = stream.slice(7, 5);
         assert_eq!(error, Err(StreamErr::SliceOutOfBounds));
 
-        let stream = Stream::from_points(&[0.0, 0.1, 0.2, 0.3, 0.4, 0.5]);
+        let stream = Stream::from_points(vec![0.0, 0.1, 0.2, 0.3, 0.4, 0.5]);
         let error = stream.slice(2, 20);
         assert_eq!(error, Err(StreamErr::SliceOutOfBounds));
     }
 
     #[test]
     fn test_looped_slice() {
-        let stream = Stream::from_points(&[0.0, 0.1, 0.2, 0.3, 0.4, 0.5]);
+        let stream = Stream::from_points(vec![0.0, 0.1, 0.2, 0.3, 0.4, 0.5]);
         assert_eq!(
             stream.looped_slice(2, 3),
             Stream::Points(vec![0.2, 0.3, 0.4])
         );
 
-        let stream = Stream::from_points(&[0.0, 0.1, 0.2, 0.3, 0.4, 0.5]);
+        let stream = Stream::from_points(vec![0.0, 0.1, 0.2, 0.3, 0.4, 0.5]);
         assert_eq!(
             stream.looped_slice(0, 8),
             Stream::Points(vec![0.0, 0.1, 0.2, 0.3, 0.4, 0.5, 0.0, 0.1])
         );
 
-        let stream = Stream::from_points(&[0.0, 0.1]);
+        let stream = Stream::from_points(vec![0.0, 0.1]);
         assert_eq!(
             stream.looped_slice(1, 7),
             Stream::Points(vec![0.1, 0.0, 0.1, 0.0, 0.1, 0.0, 0.1])
@@ -487,7 +531,7 @@ mod tests {
 
     #[test]
     fn test_from_u8() {
-        let stream = Stream::from_points(&[0, 80, 128, 220, 256u8]);
+        let stream = Stream::from_points(vec![0, 80, 128, 220, 256u8]);
         assert_eq!(
             stream,
             Stream::Points(vec![-1.0, -0.372549, 0.003921628, 0.7254902, -1.0])
@@ -496,7 +540,7 @@ mod tests {
 
     #[test]
     fn test_from_i16() {
-        let stream = Stream::from_points(&[i16::MIN + 1, -1600, 0, 2800, i16::MAX]);
+        let stream = Stream::from_points(vec![i16::MIN + 1, -1600, 0, 2800, i16::MAX]);
         assert_eq!(
             stream,
             Stream::Points(vec![-1.0, -0.048829615, 0.0, 0.08545183, 1.0])
@@ -505,7 +549,8 @@ mod tests {
 
     #[test]
     fn test_from_i32() {
-        let stream = Stream::from_points(&[i32::MIN, -1_147_483_647, 0, 1_147_483_647, i32::MAX]);
+        let stream =
+            Stream::from_points(vec![i32::MIN, -1_147_483_647, 0, 1_147_483_647, i32::MAX]);
         assert_eq!(
             stream,
             Stream::Points(vec![-1.0, -0.5343387, 0.0, 0.5343387, 1.0])
